package de.intermoves.catalogsync.pbseasymedia.jms;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.QueueConnectionFactory;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;

/**
 * Simple test class for testing the getJMSRedelivered() output for redelivered
 * message in the case of a client crash.
 * It connects to a <code>queue/ZDSjobQueue</code> that is available via JNDI on a local
 * HornetQ-server. It has been tested against a HornetQ 2.0.0.GA that is embedded into JBoss 5.1.0.
 */
public class TestRedelivered {

    /**
     * tries to receive one message
     * @param conn the connection to the server
     * @param que the queue that is used
     * @param receive a message is attempted to receive when receive is true
     * @param acknowledge the message is acknowledged if acknowledge is true
     * @param wait if wait = true then the Thread sleeps for 100 s before acknowledging
     * @throws JMSException
     */
    private static void receiveMessage(Connection conn, Queue que, boolean receive, boolean acknowledge, boolean wait) throws JMSException {
        Session s = conn.createSession(false, Session.CLIENT_ACKNOWLEDGE);
        if (receive) {
            MessageConsumer c = s.createConsumer(que);
            TextMessage tm = (TextMessage) c.receive(1000);
            if (tm != null) {
                reportMessage(tm);
                waitAcknowledge(acknowledge, wait, tm);
            } else {
                System.out.println("No message received");
            }
        } else {
            System.out.println("no receive has been attemtped, therefore no wait and no acknowledge!");
        }
    }

    /**
     *
     * @param acknowledge the message is acknowledged if acknowledge is true
     * @param wait if wait = true then the Thread sleeps for 100 s before acknowledging
     * @param tm the message that has been received
     * @throws JMSException
     */
    private static void waitAcknowledge(boolean acknowledge, boolean wait, TextMessage tm) throws JMSException {
        if (wait) {
            try {
                System.out.println("start waiting for 100 s");
                Thread.sleep(100000);
                System.out.println("waited for 100 s");
            } catch (Exception e) {
            }

        } else {
            System.out.println("not waited");
        }
        if (acknowledge) {
            tm.acknowledge();
            System.out.println("acknowledged");
        } else {
            System.out.println("not acknowledged");
        }
    }

    /**
     * reports parameters of a message
     * @param tm
     * @throws JMSException
     */
    private static void reportMessage(TextMessage tm) throws JMSException {
        System.out.println(tm.getText());
        System.out.println("JMSXDeliveryCount is supposed to exist as a property: " + tm.propertyExists("JMSXDeliveryCount"));
        System.out.println("JMSXDeliveryCount: " + tm.getIntProperty("JMSXDeliveryCount"));
        System.out.println("tm.getJMSRedelivered(): " + tm.getJMSRedelivered());
        System.out.println("toString: " + tm.toString());
    }

    /**
     * sends the message
     * @param conn
     * @param que
     * @param send
     * @throws JMSException
     */
    private static void sendMessage(Connection conn, Queue que, boolean send) throws JMSException {
        if (send) {
            Session s = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
            MessageProducer p = s.createProducer(que);
            p.setDeliveryMode(DeliveryMode.PERSISTENT);
            p.send(s.createTextMessage("HornetQ is great!"));
            s.close();
            System.out.println("message sent");
        } else {
            System.out.println("No message sent");
        }
    }

    /**
     * sends a message and receives it
     * @param send
     * @param receive
     * @param acknowledge
     * @param wait
     * @throws Exception
     */
    public static void testRedelivered(boolean send, boolean receive, boolean acknowledge, boolean wait) throws Exception {
        Connection conn = null;
        Queue que;
        try {
            InitialContext iniCtx = new InitialContext();
            ConnectionFactory tmp = (ConnectionFactory) iniCtx.lookup("/ConnectionFactory");

            System.out.println("bound!");
            QueueConnectionFactory qcf = (QueueConnectionFactory) tmp;
            conn = qcf.createQueueConnection();
            que = (Queue) iniCtx.lookup("queue/ZDSjobQueue");
            conn = qcf.createConnection();

            sendMessage(conn, que, send);

            Thread.sleep(1000);
            conn.start();
            receiveMessage(conn, que, receive, acknowledge, wait);
        } finally {
            if (conn != null) {
                conn.close();
            }
        }
    }

    /**
     *
     * The boolean flags send, receive, acknowledge, wait simulate program parameters.
     *
     * For the first start of the program the flags
     * <UL>
     * <LI>send = true,</LI>
     * <LI>receive = true,</LI>
     * <LI>acknowledge does not matter the program is terminated before</LI>
     * <LI>wait = true</LI>
     * </UL><br/>
     * are used
     *
     * Terminate the JVM when "start waiting for 100 s" is printed out.
     * <br/>
     *
     * Wait for the second run until the server has released the client connection.
     * This time depends on the TTL that is configured.<br/>
     * For the second run the parameters
     * <UL>
     * <LI>send = false,</LI>
     * <LI>receive = true,</LI>
     * <LI>acknowledge true</LI>
     * <LI>wait = false.</LI>
     * </UL>
     * are used. Simple set the boolean flag first to false and recompile
     * 
     * The redelivered message is simply consumed and the values of the parameters
     * JMSXDeliveryCount and JMSXDeliveryCount are printed on the console.
     *
     *
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {
        System.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory");
        System.setProperty(Context.PROVIDER_URL, "jnp://localhost:1099");
        System.setProperty(Context.URL_PKG_PREFIXES, "org.jboss.naming:org.jnp.interfaces");

        // FIXME: for the first run set first = true;
        // for the second run set first = false and recompile
        boolean first = true;

        boolean send;
        boolean receive;
        boolean acknowledge;
        boolean wait;
        if (first == true) {
            // is a message sent
            send = true;
            // shall a message be received
            receive = true;
            // shall the received message be acknowledged
            acknowledge = false; // acknowledgement is not reached if the JVM is terminated while waiting
            // shall waiting be activated to be able to shutdown the JVM?
            wait = true;
        } else {// second time

            send = false;
            // shall a message be received
            receive = true;
            // shall the received message be acknowledged
            acknowledge = true;
            // shall waiting be activated to be able to shutdown the JVM?
            wait = false;
        }

        System.out.println("send: " + send + " receive: " + receive + " acknowledge: " + acknowledge + " wait: " + wait);
        
        testRedelivered(send, receive, acknowledge, wait);
    }
}

